==> ui-18.1.1.tgz: package/components/menu/Sub.js <==
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useState, useRef, useImperativeHandle, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useEventCallback, useRefExtra } from '@react-devui/hooks';
import { checkNodeExist, getClassName, getHorizontalSidePosition, getVerticalSidePosition } from '@react-devui/utils';
import { useMaxIndex } from '../../hooks';
import { TTANSITION_DURING_BASE, TTANSITION_DURING_POPUP, WINDOW_SPACE } from '../../utils';
import { DPopup } from '../_popup';
import { DCollapseTransition, DTransition } from '../_transition';
import { usePrefixConfig, useTranslation } from '../root';
function Sub(props, ref) {
    const { children, dId, dLevel, dStep, dSpace, dIcon, dList, dPopupState, dTrigger, dPosinset, dMode, dInNav, dActive, dExpand, dEmpty, dFocusVisible, dDisabled, onVisibleChange, onSubClick, } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const liRef = useRef(null);
    const ulRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${dPrefix}menu-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${dPrefix}menu-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    //#endregion
    const dataRef = useRef({});
    const [t] = useTranslation();
    const isVisible = !isUndefined(dPopupState);
    const inHorizontalNav = dMode === 'horizontal' && dInNav;
    const _iconMode = dMode === 'icon' && dInNav;
    const [iconMode, setIconMode] = useState(_iconMode);
    useEffect(() => {
        setIconMode(_iconMode);
    }, [_iconMode]);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (isVisible && ulRef.current && liRef.current) {
            const height = ulRef.current.offsetHeight;
            let width = ulRef.current.offsetWidth;
            if (inHorizontalNav) {
                width = liRef.current.offsetWidth - 32;
            }
            const { top, left, transformOrigin } = inHorizontalNav
                ? getVerticalSidePosition(liRef.current, { width, height }, {
                    placement: 'bottom',
                    offset: 12,
                    inWindow: WINDOW_SPACE,
                })
                : getHorizontalSidePosition(liRef.current, { width, height }, {
                    placement: 'right',
                    offset: dInNav ? 10 : 14,
                    inWindow: WINDOW_SPACE,
                });
            setPopupPositionStyle({
                top,
                left,
                width: inHorizontalNav ? width : undefined,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    const maxZIndex = useMaxIndex(isVisible);
    useImperativeHandle(ref, () => updatePosition, [updatePosition]);
    const liNode = (_jsxs("li", Object.assign({ ref: liRef, id: dId, className: getClassName(`${dPrefix}menu__item`, `${dPrefix}menu__item--sub`, {
            [`${dPrefix}menu__item--horizontal`]: inHorizontalNav,
            [`${dPrefix}menu__item--icon`]: iconMode,
            'is-active': dActive,
            'is-expand': dMode === 'vertical' ? dExpand : isVisible,
            'is-disabled': dDisabled,
        }), style: { paddingLeft: dSpace + dLevel * dStep }, role: "menuitem", "aria-haspopup": true, "aria-expanded": dMode === 'vertical' ? dExpand : isVisible, "aria-disabled": dDisabled, onClick: (e) => {
            onSubClick(e);
        } }, { children: [dFocusVisible && _jsx("div", { className: `${dPrefix}focus-outline` }), _jsx("div", Object.assign({ className: getClassName(`${dPrefix}menu__indicator`, {
                    [`${dPrefix}menu__indicator--first`]: dPosinset[0] === 0 && dPosinset[1] > 1,
                    [`${dPrefix}menu__indicator--last`]: dPosinset[0] === dPosinset[1] - 1 && dPosinset[1] > 1,
                }) }, { children: _jsx("div", { style: { backgroundColor: dLevel === 0 ? 'transparent' : undefined } }) })), checkNodeExist(dIcon) && _jsx("div", Object.assign({ className: `${dPrefix}menu__item-icon` }, { children: dIcon })), _jsx("div", Object.assign({ className: `${dPrefix}menu__item-content` }, { children: children })), !inHorizontalNav && (_jsxs("div", Object.assign({ className: getClassName(`${dPrefix}menu__sub-arrow`, {
                    [`${dPrefix}menu__sub-arrow--horizontal`]: dMode !== 'vertical' && !inHorizontalNav,
                    'is-expand': dMode === 'vertical' && dExpand,
                }), "aria-hidden": true }, { children: [_jsx("div", {}), _jsx("div", {})] })))] })));
    return (_jsxs(_Fragment, { children: [dDisabled || dMode === 'vertical' ? (liNode) : (_jsx(DPopup, Object.assign({ dVisible: dPopupState !== null && dPopupState !== void 0 ? dPopupState : false, dTrigger: dTrigger, dUpdatePosition: {
                    fn: updatePosition,
                    triggerRef: liRef,
                    popupRef: ulRef,
                    extraScrollRefs: [],
                }, onVisibleChange: onVisibleChange }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(liNode), containerRef.current &&
                            ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: isVisible, dDuring: TTANSITION_DURING_POPUP, onEnter: updatePosition }, { children: (state) => {
                                    let transitionStyle = {};
                                    switch (state) {
                                        case 'enter':
                                            transitionStyle = { transform: inHorizontalNav ? 'scaleY(0.7)' : 'scale(0)', opacity: 0 };
                                            break;
                                        case 'entering':
                                            transitionStyle = {
                                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                                                transformOrigin,
                                            };
                                            break;
                                        case 'leaving':
                                            transitionStyle = {
                                                transform: inHorizontalNav ? 'scaleY(0.7)' : 'scale(0)',
                                                opacity: 0,
                                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                                                transformOrigin,
                                            };
                                            break;
                                        case 'leaved':
                                            transitionStyle = { display: 'none' };
                                            break;
                                        default:
                                            break;
                                    }
                                    return renderPopup(_jsx("ul", Object.assign({ ref: ulRef, className: getClassName(`${dPrefix}menu__sub-list`, `${dPrefix}menu__sub-list--popup`), style: Object.assign(Object.assign(Object.assign({}, popupPositionStyle), { minWidth: inHorizontalNav ? undefined : 160, zIndex: maxZIndex }), transitionStyle), role: "menu", "aria-labelledby": dId }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}menu__empty`, style: { paddingLeft: dSpace + dLevel * dStep } }, { children: t('No Data') }))) : (dList) })));
                                } })), containerRef.current)] })) }))), _jsx(DCollapseTransition, Object.assign({ dOriginalSize: {
                    height: 'auto',
                }, dCollapsedStyle: {
                    height: 0,
                }, dIn: dMode === 'vertical' ? dExpand : false, dDuring: TTANSITION_DURING_BASE, dStyles: {
                    entering: {
                        transition: ['height', 'padding', 'margin'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-out`).join(', '),
                    },
                    leaving: {
                        transition: ['height', 'padding', 'margin'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-in`).join(', '),
                    },
                    leaved: { display: 'none' },
                } }, { children: (collapseRef, collapseStyle, state) => {
                    const node = dMode === 'vertical' ? (_jsx("ul", Object.assign({ ref: collapseRef, className: `${dPrefix}menu__sub-list`, style: collapseStyle, role: "menu", "aria-labelledby": dId }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}menu__empty`, style: { paddingLeft: dSpace + (dLevel + 1) * dStep } }, { children: t('No Data') }))) : (dList) }))) : dInNav && state !== 'leaved' && React.isValidElement(dataRef.current.nodeCache) ? (React.cloneElement(dataRef.current.nodeCache, { style: collapseStyle })) : null;
                    if (dMode === 'vertical' && dInNav) {
                        dataRef.current.nodeCache = node;
                    }
                    return node;
                } }))] }));
}
export const DSub = React.forwardRef(Sub);

==> ui-18.3.4.tgz: package/components/menu/Sub.js <==
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useState, useRef, useImperativeHandle, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useEventCallback, useRefExtra } from '@react-devui/hooks';
import { checkNodeExist, getClassName, getHorizontalSidePosition, getVerticalSidePosition } from '@react-devui/utils';
import { useMaxIndex } from '../../hooks';
import { TTANSITION_DURING_BASE, TTANSITION_DURING_POPUP, WINDOW_SPACE } from '../../utils';
import { DPopup } from '../_popup';
import { DCollapseTransition, DTransition } from '../_transition';
import { usePrefixConfig, useTranslation } from '../root';
function Sub(props, ref) {
    const { children, dId, dLevel, dStep, dSpace, dIcon, dList, dPopupState, dTrigger, dPosinset, dMode, dInNav, dActive, dIncludeActive, dExpand, dEmpty, dFocusVisible, dDisabled, onVisibleChange, onSubClick, } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const liRef = useRef(null);
    const ulRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${dPrefix}menu-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${dPrefix}menu-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    //#endregion
    const dataRef = useRef({});
    const [t] = useTranslation();
    const isVisible = !isUndefined(dPopupState);
    const inHorizontalNav = dMode === 'horizontal' && dInNav;
    const _iconMode = dMode === 'icon' && dInNav;
    const [iconMode, setIconMode] = useState(_iconMode);
    useEffect(() => {
        setIconMode(_iconMode);
    }, [_iconMode]);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (isVisible && ulRef.current && liRef.current) {
            const height = ulRef.current.offsetHeight;
            let width = ulRef.current.offsetWidth;
            if (inHorizontalNav) {
                width = liRef.current.offsetWidth - 32;
            }
            const { top, left, transformOrigin } = inHorizontalNav
                ? getVerticalSidePosition(liRef.current, { width, height }, {
                    placement: 'bottom',
                    offset: 12,
                    inWindow: WINDOW_SPACE,
                })
                : getHorizontalSidePosition(liRef.current, { width, height }, {
                    placement: 'right',
                    offset: dInNav ? 10 : 14,
                    inWindow: WINDOW_SPACE,
                });
            setPopupPositionStyle({
                top,
                left,
                width: inHorizontalNav ? width : undefined,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    const maxZIndex = useMaxIndex(isVisible);
    useImperativeHandle(ref, () => updatePosition, [updatePosition]);
    const liNode = (_jsxs("li", Object.assign({ ref: liRef, id: dId, className: getClassName(`${dPrefix}menu__item`, `${dPrefix}menu__item--sub`, {
            [`${dPrefix}menu__item--horizontal`]: inHorizontalNav,
            [`${dPrefix}menu__item--icon`]: iconMode,
            'is-active': dActive,
            'is-expand': dMode === 'vertical' ? dExpand && dIncludeActive : isVisible,
            'is-disabled': dDisabled,
        }), style: { paddingLeft: dSpace + dLevel * dStep }, role: "menuitem", "aria-haspopup": true, "aria-expanded": dMode === 'vertical' ? dExpand : isVisible, "aria-disabled": dDisabled, onClick: (e) => {
            onSubClick(e);
        } }, { children: [dFocusVisible && _jsx("div", { className: `${dPrefix}focus-outline` }), _jsx("div", Object.assign({ className: getClassName(`${dPrefix}menu__indicator`, {
                    [`${dPrefix}menu__indicator--first`]: dPosinset[0] === 0 && dPosinset[1] > 1,
                    [`${dPrefix}menu__indicator--last`]: dPosinset[0] === dPosinset[1] - 1 && dPosinset[1] > 1,
                }) }, { children: _jsx("div", { style: { backgroundColor: dLevel === 0 ? 'transparent' : undefined } }) })), checkNodeExist(dIcon) && _jsx("div", Object.assign({ className: `${dPrefix}menu__item-icon` }, { children: dIcon })), _jsx("div", Object.assign({ className: `${dPrefix}menu__item-content` }, { children: children })), !inHorizontalNav && (_jsxs("div", Object.assign({ className: getClassName(`${dPrefix}menu__sub-arrow`, {
                    [`${dPrefix}menu__sub-arrow--horizontal`]: dMode !== 'vertical' && !inHorizontalNav,
                    'is-expand': dMode === 'vertical' && dExpand,
                }), "aria-hidden": true }, { children: [_jsx("div", {}), _jsx("div", {})] })))] })));
    return (_jsxs(_Fragment, { children: [dDisabled || dMode === 'vertical' ? (liNode) : (_jsx(DPopup, Object.assign({ dVisible: dPopupState !== null && dPopupState !== void 0 ? dPopupState : false, dTrigger: dTrigger, dUpdatePosition: {
                    fn: updatePosition,
                    triggerRef: liRef,
                    popupRef: ulRef,
                    extraScrollRefs: [],
                }, onVisibleChange: onVisibleChange }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(liNode), containerRef.current &&
                            ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: isVisible, dDuring: TTANSITION_DURING_POPUP, onEnter: updatePosition }, { children: (state) => {
                                    let transitionStyle = {};
                                    switch (state) {
                                        case 'enter':
                                            transitionStyle = { transform: inHorizontalNav ? 'scaleY(0.7)' : 'scale(0)', opacity: 0 };
                                            break;
                                        case 'entering':
                                            transitionStyle = {
                                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                                                transformOrigin,
                                            };
                                            break;
                                        case 'leaving':
                                            transitionStyle = {
                                                transform: inHorizontalNav ? 'scaleY(0.7)' : 'scale(0)',
                                                opacity: 0,
                                                transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                                                transformOrigin,
                                            };
                                            break;
                                        case 'leaved':
                                            transitionStyle = { display: 'none' };
                                            break;
                                        default:
                                            break;
                                    }
                                    return renderPopup(_jsx("ul", Object.assign({ ref: ulRef, className: getClassName(`${dPrefix}menu__sub-list`, `${dPrefix}menu__sub-list--popup`), style: Object.assign(Object.assign(Object.assign({}, popupPositionStyle), { minWidth: inHorizontalNav ? undefined : 160, zIndex: maxZIndex }), transitionStyle), role: "menu", "aria-labelledby": dId }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}menu__empty`, style: { paddingLeft: dSpace + dLevel * dStep } }, { children: t('No Data') }))) : (dList) })));
                                } })), containerRef.current)] })) }))), _jsx(DCollapseTransition, Object.assign({ dOriginalSize: {
                    height: 'auto',
                }, dCollapsedStyle: {
                    height: 0,
                }, dIn: dMode === 'vertical' ? dExpand : false, dDuring: TTANSITION_DURING_BASE, dStyles: {
                    entering: {
                        transition: ['height', 'padding', 'margin'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-out`).join(', '),
                    },
                    leaving: {
                        transition: ['height', 'padding', 'margin'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-in`).join(', '),
                    },
                    leaved: { display: 'none' },
                } }, { children: (collapseRef, collapseStyle, state) => {
                    const node = dMode === 'vertical' ? (_jsx("ul", Object.assign({ ref: collapseRef, className: `${dPrefix}menu__sub-list`, style: collapseStyle, role: "menu", "aria-labelledby": dId }, { children: dEmpty ? (_jsx("div", Object.assign({ className: `${dPrefix}menu__empty`, style: { paddingLeft: dSpace + (dLevel + 1) * dStep } }, { children: t('No Data') }))) : (dList) }))) : dInNav && state !== 'leaved' && React.isValidElement(dataRef.current.nodeCache) ? (React.cloneElement(dataRef.current.nodeCache, { style: collapseStyle })) : null;
                    if (dMode === 'vertical' && dInNav) {
                        dataRef.current.nodeCache = node;
                    }
                    return node;
                } }))] }));
}
export const DSub = React.forwardRef(Sub);
